module NonGrata
    class AuthorizationRole < ActiveRecord::Base

        self.table_name = 'non_grata_authorization_roles'

        scope :for_tenant, ->(tenant){ where(tenant_id: tenant.id).where(tenant_type: tenant.class.name.underscore) } 

        validates :name, presence: true
        
        has_many :privileges, 
            class_name: 'AuthorizationPrivilege', 
            foreign_key: 'non_grata_authorization_roles_id',
            :dependent => :destroy
        
        def can?(privilege, resource) 
            privileges.exists?(privilege_name: privilege)
        end

        def grant(privilege_name, resource_name)
            privilege = privileges.find_by(privilege_name: privilege_name, resource_name: resource_name)
            return privilege if privilege
            privileges.create(privilege_name: privilege_name, resource_name: resource_name)
        end
        
        def forbid(privilege_name, resource_name)
            privilege = privileges.find_by(privilege_name: privilege_name, resource_name: resource_name)
            privilege.destroy if privilege
        end 

        def forbid_all
            privileges.each(&:destroy)
        end
        def scheme 
            NonGrata::Authorization.schemes(self.scheme_name)
        end

    end
end

