module NonGrata
    class Scheme
        attr_accessor :style, :tenantable
        def  initialize(scheme_name='main', options={})
            @name = scheme_name 
            self.style  = :declaritive

            options.each do |k,v| 
                send("#{k.to_s}=",v) if self.respond_to?("#{k}=")
            end
        end
        def is_declaritive?
            self.style == :declaritive
        end
        def is_dynamic?
            self.style == :db
        end
        ###
        # returns a list of privileges for this scheme
        # 
        def privileges
            @privileges ||= []
        end

        def name
            @name
        end

        def name=(value)
            @name = value.to_sym
        end
        
        ###
        # returns a list of roles for this scheme or returns a 
        # specific role by name
        # Ex:
        #   scheme.roles  <- returns an array of all roles
        #   scheme.roles(:user)  <- returns the user role 
        #
        def roles(role_name = nil)
            @roles ||= []
            return @roles if role_name.nil?
            return @roles.find{|i| i.name == role_name} 
        end
        
        ### 
        # this function is used by the configuration DSL
        # and should not be called directly. 
        # adds a new privilege to the scheme
        #
        def privilege(resource, name)
            if self.style == :declaritive 
                raise "Can not set privileges on a declaritive scheme. Privileges must be inside a role."
            end
            privileges << Privilege.new(resource, name)
        end 

        ###
        # this function is used by the configuration DSL
        # and should not be called directly. It is used
        # to add a new role to the scheme
        #
        def role(name, &block)
            r = Role.new(name)
            r.config(&block)
            roles << r
        end

        ###
        # called by the configuration DSL in order to parse
        # a scheme block.
        #
        # example: 
        #   scheme.config do 
        #       role :user do 
        #           privilege :site, :login
        #       end
        #   end
        def config(&block)
            instance_eval(&block)
        end

    end
end

